@@ -6,6 +6,7 @@ Created on Sat Jun  1 15:11:33 2019
 @author: Gabriele Coiana
 """
 import numpy as np
+from scipy import signal
 from decomp import  read,  util#,  plot 
 import sys
 
@@ -81,6 +82,8 @@ print('\nDone. Performing decomposition...\n')
 #anis = list(range(tot_atoms_uc*3))
 #namedir = plot.create_folder(system)
 #every_tot = int(tot_atoms_conventional_cell/tot_atoms_uc)
+Num_steps = util.odd_num_steps(Num_timesteps) #this is very stupid: it is to have a odd number of steps 
+
 for i in range(Nqpoints):
     eigvec = eigvecs[i]
     eigvecH = np.conjugate(eigvec.T)
@@ -114,14 +117,23 @@ for i in range(Nqpoints):
     
     Params = np.zeros((3,tot_atoms_uc*3))
     for n in range(tot_atoms_uc*3):
-        x_data, y_data = freq[0:int(Num_timesteps/2)], Sq_proj[0:int(Num_timesteps/2),n]
+        x_data, y_data = freq[0:int((Num_steps-1)/2)], Sq_proj[0:int((Num_steps-1)/2),n]
         popt, perr = util.fit_to_lorentzian(x_data, y_data, k, n)
+        
+        #now you go apply to Voigt equation for the sigmas
+        sig, mi = popt[-1]/(2*np.sqrt(2*np.log(2))),  x_data[int(len(x_data)/2)]
+        gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((x_data-mi)/sig)**2)                
+        convolution = signal.fftconvolve(y_data,gaussian, mode='same') / sum( gaussian)
+        omega_l, FWHM_l = util.voigt_eq(n, k, freq, convolution, sig)
+        
         print()
         print('\t\tFitting to Lorentzian, mode '+str(n)+'...')
-        print('\t\tResonant frequency omega =',np.round(popt[0],2),' +-',np.round(perr[0],3))
+        print('\t\tResonant frequency Lorentzian =',np.round(popt[0],2),' +-',np.round(perr[0],3))
+        print('\t\tResonant frequency Convolution =',np.round(omega_l,2))
         print('\t\tLinewidth gamma =',np.round(popt[2],2),' +-',np.round(perr[2],3))
+        print('\t\tFWHM Convolution =',np.round(FWHM_l,2))
         print()
-        Params[:,n] = popt
+        Params[:,n] = [omega_l, popt[1], FWHM_l]
 
 #        anis[n] = plot.plot_with_ani(freq[0:meta],Z[0:meta,n],Z_q[0:meta], k, eigvec[:,n],freq_disp[n],n,Ruc,file_eigenvectors,masses_for_animation)
 #        plot.save_proj(ff[0:meta*2],Sq_proj[0:meta*2,n],Sq[0:meta*2], qpoints_scaled[i], Ruc, eigvec[:,n],freq_disp[n],n,namedir,masses_for_animation, popt, -1)  
