@@ -25,7 +25,7 @@ def figsize(scale):
 import warnings
 warnings.filterwarnings('ignore') 
 
-
+from scipy.interpolate import interp1d
 import matplotlib as mpl
 mpl.use('pgf')
 import matplotlib.pyplot as plt
@@ -34,6 +34,7 @@ import matplotlib.gridspec as gridspec
 from matplotlib import animation
 from scipy import interpolate
 import os
+from decomp import util
 
 
 
@@ -115,6 +116,8 @@ def plot_eigvec_noani(Ruc, n_uc, k,eigvec,freq, ax,fig, masses):
     EIG_exp = np.multiply(eigvec,exp1)
     print('Displacement components:')
     print(np.round(np.reshape(np.real(EIG_exp),np.shape(Ruc)),2))
+    print('# =============================================================================')
+
     print()
     
     Ruc = Ruc.flatten()
@@ -166,7 +169,6 @@ def plot6(x,y,ytot, k,eigvec,freq,n,Ruc,eigname,masses, max_Z):
 
     }
     mpl.rcParams.update(pgf_with_latex)
-
     
     fig,ax = plt.subplots()
 #    graph2, = ax.plot(x,ytot,'--',label='qpoint DOS', color='orangered')
@@ -192,7 +194,9 @@ def plot6(x,y,ytot, k,eigvec,freq,n,Ruc,eigname,masses, max_Z):
     ax2.set_xlabel('x'), ax2.set_ylabel('y'), ax2.set_zlabel('z')
     ampl = 10
     eigvec = eigvec/np.sqrt(masses)*ampl
-    print('Mode frequency: ', freq, '\n')
+
+    print('# =============================================================================')
+    print('Kpoint '+str(np.round(k,2))+' Mode '+str(n)+'; frequency: ', freq, '\n')
     print('Eigenvector components:')
     print(np.round(np.reshape(np.real(eigvec),np.shape(Ruc)),2))
     print()
@@ -340,7 +344,11 @@ def save_proj(x,y,ytot,convolution, k,Ruc, eigvec,freq,n,namedir,masses, params,
     plt.close(fig1)
     return 
 
-def plot_k2(f, data, data_projected, indexes,freqs, max_Z, branches,  freq_res, title=''):
+
+    
+
+
+def plot_k2(f, data, data_projected, indexes,freqs, max_Z, branches, ks_path, title=''):
     pgf_with_latex = {                      # setup matplotlib to use latex for output
 
     "pgf.texsystem": "pdflatex",        # change this if using xetex or lautex
@@ -376,46 +384,65 @@ def plot_k2(f, data, data_projected, indexes,freqs, max_Z, branches,  freq_res,
     }
 
     mpl.rcParams.update(pgf_with_latex)
-    N = len(data[0,:])
+    num_ks = len(data_projected[0,0,:])
     tot_branches = int(len(freqs[0][0,:]))
 
-    kk = np.linspace(0,1,N)
+    kk = np.linspace(0,1,num_ks)
     X, Y = np.meshgrid(kk,f)
+    
     fig, ax = plt.subplots()
     
-    if(len(branches) == tot_branches): #plot whole spectrum
-        if(max_Z > np.max(data) or max_Z < 0.0):
-            max_Z = np.max(data)
-        print('Drawing the whole spectrum...')
-        ax.contourf(X,Y,data,100, vmax=max_Z)
-    else:
-        if(max_Z > np.max(data_projected) or max_Z < 0.0):
-            max_Z = np.max(data_projected)
-        print('Drawing branches ', branches)
-        ax.contourf(X,Y,np.sum(data_projected[:,:,:], axis=1),100, vmax=max_Z)
     
-    plt.xticks(kk,indexes)
+#    if(len(branches) == tot_branches): #plot whole spectrum
+#        if(max_Z > np.max(data) or max_Z < 0.0):
+#            max_Z = np.max(data)
+#        print('Drawing the whole spectrum...')
+#        ax.contourf(X,Y,data,100, vmax=max_Z)
+#        print('finished')
+
+    if(max_Z > np.max(data_projected) or max_Z < 0.0):
+        max_Z = np.max(data_projected)
+    print('Drawing branches ', branches)
+    ax.contourf(X,Y,np.sum(data_projected, axis=1),100, vmax=max_Z)
+    print('finished')
+        
+    num_Gammas, indexes_Gammas, indexes_Gammas_withboundaries = util.find_Gammas(ks_path)
+    
+    
     
     ax2 = plt.twinx()
     ax2.set_ylabel('[Thz]')
     num_dispersions = len(freqs)
     colours = ['black', 'red']
-    for l in range(num_dispersions):
-        for j in (branches):
-            graph, = ax2.scatter(kk,freqs[l][:,j], marker='x', c=colours[l], label=title[l])
+    graphs = [0,0]
+    num_ks_commensurate = len(freqs[0][:,0])
+    kk_commensurate = np.linspace(0,1,num_ks_commensurate)
+    for l in range(1):#(num_dispersions):
+        for j in branches:
+           for m in range(len(indexes_Gammas)):
+                this_G = indexes_Gammas_withboundaries[m]
+                next_G = indexes_Gammas_withboundaries[m+1]+1
+                kk_i = kk_commensurate[this_G:next_G]
+                f_i = freqs[l][this_G:next_G,j]
+                interp = interp1d(kk_i, f_i, 'cubic')
+                x_i = np.arange(kk_i[0], kk_i[-1], 0.001)
+                graphs[l] = ax2.plot(x_i,interp(x_i),  c=colours[l], label=title[l])
+           ax2.scatter(kk_commensurate, freqs[l][:,j], c='red')
+
+           
     ax2.get_shared_y_axes().join(ax, ax2)
     plt.ylim([f[0], f[-1]])
     plt.xlim([kk[0], kk[-1]])
     
-    handles, labels = ax.get_legend_handles_labels()
-    plt.legend(handles=[graph])
+    plt.xticks(kk_commensurate,indexes)
+    plt.legend(handles=(graphs[0][0],))#graphs[1][0]
     ax.tick_params(axis='both', which='major', labelsize=15, width=2 , length=6, direction='in', top=True, left=True, bottom=True)
     ax2.tick_params(axis='y', which='major', labelsize=15, width=2 , length=6, direction='in')
     plt.rcParams['axes.linewidth'] = 2
 #    plt.show()
     plt.suptitle('Spectrum branches '+str(branches))
     fig.savefig('plot5_'+str(branches)+'.pdf')
-    return
+    return 
 
 
 def plot_k(f, data, data_projected, indexes,freqs, max_Z, branches,  freq_res, title=''):
