@@ -9,29 +9,30 @@ Created on Fri Jan 24 14:40:14 2020
 import numpy as np
 from scipy import signal
 import os, sys
-from decomp import plot, read
+from decomp import plot, read, util
 
 
 ## =============================================================================
 ## Parameters
-input_file = sys.argv[1]
+input_file =  sys.argv[1]
 plot_types = read.read_post_proc(input_file)[0]
 n_atom_unit_cell = read.read_post_proc(input_file)[1]
 Masses = read.read_post_proc(input_file)[2]
 file_eigenvectors = read.read_post_proc(input_file)[3]
 file_SPOSCAR = read.read_post_proc(input_file)[4]
 max_Z = read.read_post_proc(input_file)[5]
-freq_res = read.read_post_proc(input_file)[6]
-kinput = read.read_post_proc(input_file)[7::][0]
-labels = read.read_post_proc(input_file)[8]
-modes = read.read_post_proc(input_file)[9]
-temperatures_folders = read.read_post_proc(input_file)[10]
+gaussian_smoothing = read.read_post_proc(input_file)[6]
+interp = read.read_post_proc(input_file)[7]
+kinput = read.read_post_proc(input_file)[8::][0]
+labels = read.read_post_proc(input_file)[9]
+modes = read.read_post_proc(input_file)[10]
+temperatures_folders = read.read_post_proc(input_file)[11]
 
 
 
 tot_atoms_uc = int(np.sum(n_atom_unit_cell)) 
 Nqpoints, qpoints_scaled, ks, freqs_disp, eigvecs_phonopy, distances, Hk = read.read_phonopy(file_eigenvectors, tot_atoms_uc)
-SCell, Ruc, R0, masses, masses_for_animation = read.read_SPOSCAR_and_masses(file_SPOSCAR, n_atom_unit_cell, n_atom_unit_cell, Masses)  
+SCell, Ruc, R0, masses, masses_uc = read.read_SPOSCAR_and_masses(file_SPOSCAR, n_atom_unit_cell, n_atom_unit_cell, Masses)  
 #### =============================================================================
 
 for mode in plot_types:
@@ -47,19 +48,30 @@ for mode in plot_types:
             to_skip2 = 4*i
             Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)
             
-    
             k_scaled = qpoints_scaled[i]
             eigvec = eigvecs_phonopy[i]
             freq_disp = freqs_disp[i]
             print(' Creating plots for k point = ', k_scaled)
             for n in range(tot_atoms_uc*3):
                 params = Params[:,n]
+                
+                if(gaussian_smoothing != 0):
+                    if(gaussian_smoothing < 0):
+                        sig, mi = params[-1]/2, freq[int(len(freq)/2)]
+                    else:
+                        sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
+                if(sig==0):
+                    sig = 0.0001
 
-                sig, mi = params[-1]/2, freq[int(len(freq)/2)]
-                gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2)
+                gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2)                
                 convolution_lorentzian = signal.fftconvolve(Z[:,n], plot.lorentzian(freq, freq[int(len(freq)/2)], params[1], params[2]), mode='same') / sum( plot.lorentzian(freq, *params))
                 convolution = signal.fftconvolve(Z[:,n],gaussian, mode='same') / sum( gaussian)
-                plot.save_proj(freq,Z[:,n],Z_q, convolution, qpoints_scaled[i], Ruc, eigvec[:,n],freq_disp[n],n,namedir,masses_for_animation, params, max_Z)
+                
+                #now you go apply to Voigt equation for the sigmas
+                fmax, fl = util.voigt_eq(freq, convolution, sig)
+                params[-1] = fl
+                
+                plot.save_proj(freq,Z[:,n],Z_q, convolution, qpoints_scaled[i], Ruc, eigvec[:,n],freq_disp[n],n,namedir,masses_uc, params, max_Z)
     
     
     if(mode==1):
@@ -156,29 +168,74 @@ for mode in plot_types:
         frequencies_disp = np.zeros((1,tot_atoms_uc*3,len(ks_path)))
         ZQS = np.zeros((meta, 1+len(ks_path))) #for the DOS
         
-        count_DOS, count = 0, 0
+        count_DOS, count_k = 0, 0
         accepted_qpoints = []
-        for i in range(Nqpoints):
+        for i in range(len(ks_path)):
+            to_add_nac, omegas_nac = False, np.zeros((2,tot_atoms_uc*3))
             to_skip = (4)*i
+            to_skip_DOS = (1+meta)*i
             qpoint = np.genfromtxt('quasiparticles',skip_header=to_skip+0, max_rows=1)
             index_disp = np.min((np.argwhere(np.all(np.equal(qpoint, qpoints_scaled), axis=1))))
-            if(np.any(np.all(np.equal(qpoint,ks_path), axis=1))): # e' un delirio questo ahaha, verifica che il qpoint trovato sia nel kpath
+            
+            #NAC correction WRONG
+            if(np.allclose(qpoint, [0,0,0])):
+                eps_inf, BORN = np.genfromtxt('../BORN', skip_header=1, max_rows=1).reshape(3,3), np.genfromtxt('../BORN', skip_header=2, max_rows=tot_atoms_uc).reshape(tot_atoms_uc*3,3)
+                a1, a2, a3 = SCell[0]/10, SCell[1]/10, SCell[2]/10
+                Vuc = np.dot(a1, np.cross(a2,a3))
+                qpoint_nac = qpoints_scaled[i+1]
+                kpoint_nac = np.dot(Hk,qpoint_nac)
+                omegas_nac, D = util.get_nac_wrong(kpoint_nac, Vuc, BORN, eps_inf, masses_uc)
+                
+                thisq_eigvec = eigvecs_phonopy[i]
+                lamdas = np.diag(freqs_disp[0]**2)
+                Dq = np.dot(thisq_eigvec, np.dot(lamdas, np.linalg.inv(thisq_eigvec)))
+                eigvals, eigvec = np.linalg.eig(D)
+#                print(np.round(np.sqrt(eigvals),2))
+                DD = D + Dq
+                eigvals, eigvec = np.linalg.eig(DD)
+#                print(np.round(np.sqrt(eigvals),2))
+                
+                for k in range(tot_atoms_uc*3):
+                    if (np.isnan(omegas_nac[k])):
+                        omegas_nac[k] = 0
+                omegas_nac = np.vstack((omegas_nac,np.zeros(tot_atoms_uc*3)))
+                to_add_nac = True
+
+            if(np.any(np.all(np.equal(qpoint,ks_path[i]))) ): # e' un delirio questo ahaha, verifica che il qpoint trovato sia nel kpath
+                accepted_qpoints.append(qpoint.tolist())
                 params = np.genfromtxt('quasiparticles',skip_header=to_skip+1, max_rows=3)
                 omegas_gammas = params[0::2,:]
-                frequencies[:,:,count] = omegas_gammas
-                frequencies_disp[:,:,count] = freqs_disp[index_disp,:]
-                count = count + 1
+                frequencies[:,:,count_k] = omegas_gammas + omegas_nac
+                frequencies_disp[:,:,count_k] = freqs_disp[index_disp,:] + omegas_nac[0,:]
+                count_k = count_k + 1
             
-            to_skip_DOS = (1+meta)*i
-            for j in range(len(ks_path)):
-                if(np.allclose(qpoint, ks_path[j]) and (qpoint.tolist() not in accepted_qpoints)):
-                    accepted_qpoints.append(qpoint.tolist())
-                    Zq = np.genfromtxt('Zqs',skip_header=to_skip_DOS+1, max_rows=meta)
-                    ZQS[:,count_DOS+1] = Zq[:,0]
-                    count_DOS = count_DOS + 1
-                
-        ZQS[:,0] = np.loadtxt('ZS', usecols=0)    
+        
+        if (interp>0):
+            num_ks_in_between = interp
+            ZQS_proj = util.interpol_spectrum(ks_path, freq, frequencies[0,:,:].T, eigvecs_phonopy, modes, np.zeros((meta, tot_atoms_uc*3, len(qpoints_scaled))), num_ks_in_between)
+        
+        #this is for the DOS
+        ZQS = np.loadtxt('ZS', usecols=(0,1))
+        
+        if(gaussian_smoothing != 0):
+            if(gaussian_smoothing < 0):
+                print('automatic sigma not allowed in this mode, sigma set to 0.1')
+                sig, mi = 0.1, freq[int(len(freq)/2)]
+            else:
+                sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
+        
+        gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2)                
+        convolution = signal.fftconvolve(ZQS[:,1],gaussian, mode='same') / sum( gaussian)
+        ZQS[:,1] = convolution    
+        
+        #this is to reshape spectra according to real branches
+        indexes, freqs_reshaped = util.get_real_branches(frequencies[0,:,:], eigvecs_phonopy, masses_uc, Hk, qpoints_scaled)
+        indexes, freqs_disp_reshaped = util.get_real_branches(frequencies_disp[0,:,:], eigvecs_phonopy, masses_uc, Hk, qpoints_scaled)
+        frequencies[0,:,:] = freqs_reshaped
+        frequencies_disp[0,:,:] = freqs_disp_reshaped
+        
         plot.plot4(distances, frequencies, frequencies_disp, ks_path, x_labels, ZQS)
+
     
     if(mode==5):
         # take the path the user inputs and find all the k points you can plot
@@ -189,12 +246,17 @@ for mode in plot_types:
         freq = np.loadtxt('ZS', usecols=0)
         meta = len(freq)
         ZQS = np.zeros((meta, len(qpoints_scaled)))
-        ZQS_proj = np.zeros((meta, num_modes, len(qpoints_scaled)))
+        ZQS_proj = np.zeros((meta, tot_atoms_uc*3, len(qpoints_scaled)))
         count = 0
         accepted_qpoints = []
         freqs_from_disp = np.zeros((len(qpoints_scaled),tot_atoms_uc*3))
+        
         for i in range(Nqpoints):
+            to_add_nac = False
             to_skip = (1+meta)*i
+            to_skip2 = 4*i
+            Params = np.genfromtxt('quasiparticles',skip_header=to_skip2+1, max_rows=3)
+            
             qpoint = np.genfromtxt('Zqs',skip_header=to_skip+0, max_rows=1)
             for j in range(len(qpoints_scaled)):
                 if(np.allclose(qpoint, qpoints_scaled[j]) and ((qpoint.tolist() not in accepted_qpoints) or i==j)): #è un completo delirio
@@ -203,15 +265,69 @@ for mode in plot_types:
                         if(np.allclose(qpoint,qpoints_scaled[m])):
                             freqs_from_disp[count, :] = freqs_disp[m,:]
                             break
+                    
+                    #NAC correction WRONG
+                    if(np.allclose(qpoint, [0,0,0])):
+                        eps_inf, BORN = np.genfromtxt('../BORN', skip_header=1, max_rows=1).reshape(3,3), np.genfromtxt('../BORN', skip_header=2, max_rows=tot_atoms_uc).reshape(tot_atoms_uc*3,3)
+                        a1, a2, a3 = SCell[0]/10, SCell[1]/10, SCell[2]/10
+                        Vuc = np.dot(a1, np.cross(a2,a3))
+                        qpoint_nac = qpoints_scaled[j+1]
+                        kpoint_nac = np.dot(Hk,qpoint_nac)
+                        omegas_nac, D = util.get_nac_wrong(kpoint_nac, Vuc, BORN, eps_inf, masses_uc)
+                        to_add_nac = True
+                    
+
                     Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
-                    if(num_modes == tot_atoms_uc*3):
-                        ZQS[:,count] = Zq[:,0]
-                    else:
-                        for b in range(len(modes)):
-                            mm = modes[b]
-                            ZQS_proj[:,b,i] = Zq[:,mm+1]
-                    count = count + 1
-        plot.plot_k2(freq,ZQS,ZQS_proj, x_labels,[freqs_from_disp],max_Z,modes, freq_res, title=['0K dispersion'])
+                    
+                    ZQS[:,count] = Zq[:,0]
+                    for b in range(tot_atoms_uc*3):
+                        params = Params[:,b]
+                        if (to_add_nac):
+                            f_shift = omegas_nac[b]
+                            if (np.isnan(f_shift) or np.isclose(f_shift,0, atol=0.1)):
+                                aaa = 0
+                            else:
+                                to_shift = int(np.argwhere(freq <= f_shift)[-1])
+                                adding_zeros = np.zeros((to_shift))
+                                Zq[0:to_shift, b+1] = adding_zeros
+                                Zq[to_shift::, b+1] = Zq[0:meta-to_shift, b+1]
+
+                                freqs_from_disp[count, b] = freqs_from_disp[count, b] + f_shift
+#                                fig,ax = plt.subplots()
+#                                ax.plot(freq, Zq[:,:])
+
+                        if (params[-1] == 0):
+                            sig = 0.0001
+                             
+                        if(gaussian_smoothing != 0):
+                            if(gaussian_smoothing < 0):
+                                sig, mi = params[-1]/2, freq[int(len(freq)/2)]
+                            else:
+                                sig, mi = gaussian_smoothing, freq[int(len(freq)/2)]
+                            gaussian = 1/(sig*np.sqrt(2*np.pi)) * np.exp(-.5*((freq-mi)/sig)**2)
+                            convolution = signal.fftconvolve(Zq[:,b+1],gaussian, mode='same') / sum( gaussian)
+                            ZQS_proj[:,b,i] = convolution
+                        else:
+                            ZQS_proj[:,b,i] = Zq[:,b+1]
+                        
+            count = count + 1
+
+        #this is to reshape spectra according to real branches
+        indexes, freqs_reshaped = util.get_real_branches(freqs_from_disp.T, eigvecs_phonopy, masses_uc, Hk, qpoints_scaled)
+        ZQS_proj_reshaped = np.zeros((meta, num_modes, len(qpoints_scaled)))
+        for j in range(len(ks)):
+            ZQS_proj_reshaped[:,:,j] = ZQS_proj[:,indexes[j,modes],j]
+        
+        freqs_from_spectrum = util.freqs_spectrum(freq,ZQS_proj_reshaped)
+        
+
+        if (interp>0):
+            num_ks_in_between = interp
+            ZQS_proj = util.interpol_spectrum(ks_path, freq, freqs_from_spectrum, eigvecs_phonopy, modes, ZQS_proj_reshaped, num_ks_in_between)
+        else:
+            ZQS_proj = ZQS_proj_reshaped[:,range(len(modes)),:]
+        
+        plot.plot_k2(freq,ZQS,ZQS_proj, x_labels,[freqs_reshaped.T, freqs_from_spectrum],max_Z,modes, ks_path, title=['0K dispersion', 'finite K dispersion'])
 
     if(mode==6):
         # take the path the user inputs and find all the k points you can plot
@@ -233,7 +349,7 @@ for mode in plot_types:
                     Zq = np.genfromtxt('Zqs',skip_header=to_skip+1, max_rows=meta)
                     for b in range(len(modes)):
                         mm = modes[b]
-                        ani = plot.plot6(freq,Zq[:,mm+1],Zq[:,0], np.dot(Hk,qpoint),eigvec[:,mm],freq_disp[mm],mm,Ruc,file_eigenvectors,masses_for_animation, max_Z)
+                        ani = plot.plot6(freq,Zq[:,mm+1],Zq[:,0], np.dot(Hk,qpoint),eigvec[:,mm],freq_disp[mm],mm,Ruc,file_eigenvectors,masses_uc, max_Z)
         
         
         
@@ -300,7 +416,7 @@ for mode in plot_types:
                 print()
                 print('freqs: ', np.round(freqs,3), ' taken ', np.round(freq,3))
                 print('\neigenvec:')
-                print(np.round(eigvec*ampl/np.sqrt(masses_for_animation),2).reshape(np.shape(Ruc)))
+                print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
                 print('# =============================================================================')
                 print()
                 C11 = rho * (freq/csi)**2 *conv_factor
@@ -322,7 +438,7 @@ for mode in plot_types:
                 print()
                 print(np.round(freqs,3), ' taken ', np.round(freq,3))
                 print('\neigvec:')
-                print(np.round(eigvec*ampl/np.sqrt(masses_for_animation),2))
+                print(np.round(eigvec*ampl/np.sqrt(masses_uc),2))
                 C44_1 = rho * (freq/csi)**2 *conv_factor
                 print('# =============================================================================')
                 print()
@@ -356,7 +472,7 @@ for mode in plot_types:
                 print()
                 print('freqs: ', freqs, ' taken ', freq)
                 print('\neigenvec:')
-                print(np.round(eigvec*ampl/np.sqrt(masses_for_animation),2).reshape(np.shape(Ruc)))
+                print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
                 print('# =============================================================================')
                 print()
                 C44_2 = .5* rho * (freq/csi)**2 *conv_factor
@@ -381,7 +497,7 @@ for mode in plot_types:
                 print()
                 print(np.round(freqs,3), ' taken ', np.round(freq,3))
                 print('\neigvec:')
-                print(np.round(eigvec*ampl/np.sqrt(masses_for_animation),2))
+                print(np.round(eigvec*ampl/np.sqrt(masses_uc),2))
                 A = rho * (freq/csi)**2 *conv_factor
                 C12_1 =C11-A  #<- with freqs[0] # A - C11 -2*C44 #
                 print('# =============================================================================')
@@ -406,7 +522,7 @@ for mode in plot_types:
                 print()
                 print('freqs: ', freqs, ' taken ', freq)
                 print('eigenvec:')
-                print(np.round(eigvec*ampl/np.sqrt(masses_for_animation),2).reshape(np.shape(Ruc)))
+                print(np.round(eigvec*ampl/np.sqrt(masses_uc),2).reshape(np.shape(Ruc)))
                 print('# =============================================================================')
                 print()
                 A = rho * (freq/csi)**2 *conv_factor
