@@ -86,21 +86,27 @@ def read_post_proc(input_file):
     
     max_Z = float(lista[6])
     
-    freq_res = float(lista[7])
+    gaussian_smoothing = lista[7]
+    if(gaussian_smoothing=='auto'):
+        gaussian_smoothing = -1
+    else:
+        gaussian_smoothing = float(lista[7])
     
-    ks = lista[8]
+    interp = int(lista[8])
+    
+    ks = lista[9]
     a = np.fromstring(ks, dtype=np.float, sep=',')
     num = len(a)/3
     kpoints = np.split(a,num)
     
-    labs = lista[9]
+    labs = lista[10]
     labels = labs.split(',')
     
-    modes = np.fromstring(lista[10], dtype=np.int, sep=',')
+    modes = np.fromstring(lista[11], dtype=np.int, sep=',')
     
-    temperature_folders = lista[11]
+    temperature_folders = lista[12]
     
-    return plot_types, n_atom_unit_cell, Masses, file_eigenvectors, file_SPOSCAR, max_Z, freq_res, kpoints, labels, modes, temperature_folders
+    return plot_types, n_atom_unit_cell, Masses, file_eigenvectors, file_SPOSCAR, max_Z, gaussian_smoothing, interp, kpoints, labels, modes, temperature_folders
 
 
 
